﻿// <copyright file="DateComparer.cs" company="GoalBook"> 
//    Copyright © 2009 Mark Brownsword. All rights reserved.
//    This source code and supporting files are licensed under The Code Project  
//    Open License (CPOL) as detailed at http://www.codeproject.com/info/cpol10.aspx. 
// </copyright>
namespace GoalBook.Infrastructure.Comparers
{
    using System.Collections;
    using System;

    /// <summary>
    /// Date Comparer.
    /// </summary>
    public class TimeInfoComparer : IComparer
    {
        /// <summary>
        /// Compare implementation.
        /// </summary>
        /// <param name="x">x parameter</param>
        /// <param name="y">y parameter</param>
        /// <returns>        
        /// Value               Condition                
        /// Less than zero      x is less than y
        /// Zero                x equals y
        /// Greater than zero   x is greater than y
        /// </returns>
        public int Compare(object x, object y)
        {
            if (x == y)
            {
                return 0;
            }

            if (x == null || string.IsNullOrEmpty(x.ToString()))
            {
                return 1;
            }

            if (y == null || string.IsNullOrEmpty(y.ToString()))
            {
                return -1;
            }

            TimeInfo t1;
            TimeInfo t2;
            TimeInfo.TryParse((string)x, out t1);
            TimeInfo.TryParse((string)y, out t2);

            DateTime dateTime1 = DateTime.Parse(string.Concat(DateTime.Today.ToShortDateString(), " ", t1.ToString()));
            DateTime dateTime2 = DateTime.Parse(string.Concat(DateTime.Today.ToShortDateString(), " ", t2.ToString()));

            return DateTime.Compare(dateTime1, dateTime2);
        }            
    }
}
