﻿// <copyright file="XamlToFlowDocumentConverter.cs" company="GoalBook"> 
//    Copyright © 2009 Mark Brownsword. All rights reserved.
//    This source code and supporting files are licensed under The Code Project  
//    Open License (CPOL) as detailed at http://www.codeproject.com/info/cpol10.aspx. 
// </copyright>
namespace GoalBook.Infrastructure.Converters
{
    #region Using Statements
    using System;
    using System.Globalization;
    using System.IO;
    using System.Windows.Data;
    using System.Windows.Documents;
    using System.Windows.Markup;
    using System.Xml;
    #endregion

    /// <summary>
    /// Value Converter for object model item foreign keys.
    /// </summary>
    public class XamlToFlowDocumentConverter : IValueConverter
    {
        #region IValueConverter Members
        /// <summary>
        /// Convert Xaml string To FlowDocument.
        /// </summary>
        /// <param name="value">value parameter</param>
        /// <param name="targetType">targetType parameter</param>
        /// <param name="parameter">object parameter</param>
        /// <param name="culture">culture parameter</param>
        /// <returns>FlowDocument from Toodledo string</returns>      
        public object Convert(object value, Type targetType, object parameter, CultureInfo culture)
        {
            if (string.IsNullOrEmpty(value.ToString()))
            {
                return new FlowDocument();
            }

            FlowDocument document = null;
                        
            // Load the Xaml string into FlowDocument.
            using (StringReader stringReader = new StringReader(value.ToString()))
            {
                using (XmlTextReader xmlTextReader = new XmlTextReader(stringReader))
                {
                    document = XamlReader.Load(xmlTextReader) as FlowDocument;
                }
            }

            return document;
        }

        /// <summary>
        /// Convert FlowDocument To Xaml string.
        /// </summary>
        /// <param name="value">value parameter</param>
        /// <param name="targetType">targetType parameter</param>
        /// <param name="parameter">object parameter</param>
        /// <param name="culture">culture parameter</param>
        /// <returns>Html string from FlowDocument</returns>
        public object ConvertBack(object value, Type targetType, object parameter, CultureInfo culture)
        {            
            return XamlWriter.Save(value);
        }

        #endregion
    }
}
