﻿// <copyright file="PageDefinition.cs" company="GoalBook"> 
//    Copyright © 2009 Mark Brownsword. All rights reserved.
//    This source code and supporting files are licensed under The Code Project  
//    Open License (CPOL) as detailed at http://www.codeproject.com/info/cpol10.aspx. 
// </copyright>
namespace GoalBook.Infrastructure.Printing
{
    #region Using Statements
    using System.Windows;
    using System.Windows.Media;
    #endregion

    /// <summary>
    /// PageDefinition. Defines properties for printed page.
    /// </summary>    
    public class PageDefinition
    {
        /// <summary>
        /// Declaration for headerHeight.
        /// </summary>
        private double headerHeight;

        /// <summary>
        /// Declaration for footerHeight.
        /// </summary>
        private double footerHeight;
                
        /// <summary>
        /// Gets or sets PageSize.
        /// </summary>
        public Size PageSize { get; set; }
                                
        /// <summary>
        /// Gets or sets HeaderHeight.
        /// </summary>
        public double HeaderHeight
        {
            get
            {
                if (this.headerHeight < 0)
                {
                    return 0;
                }

                return this.headerHeight;
            }

            set 
            {
                this.headerHeight = value; 
            }
        }

        /// <summary>
        /// Gets or sets FooterHeight.
        /// </summary>
        public double FooterHeight
        {
            get
            {
                if (this.footerHeight < 0)
                {
                    return 0;
                }

                return this.footerHeight;
            }

            set
            {
                this.footerHeight = value;
            }
        }

        /// <summary>
        /// Gets or sets HeaderTemplate.
        /// </summary>
        public string HeaderTemplate { get; set; }
                        
        /// <summary>
        /// Gets or sets FooterTemplate.
        /// </summary>
        public string FooterTemplate { get; set; }
        
        /// <summary>
        /// Gets or sets HeaderPadding.
        /// </summary>
        public Thickness HeaderPadding { get; set; }

        /// <summary>
        /// Gets or sets FooterPadding.
        /// </summary>
        public Thickness FooterPadding { get; set; }

        /// <summary>
        /// Gets or sets Header FontFamily.
        /// </summary>
        public FontFamily HeaderFontFamily { get; set; }

        /// <summary>
        /// Gets or sets Footer FontFamily.
        /// </summary>
        public FontFamily FooterFontFamily { get; set; }

        /// <summary>
        /// Gets or sets Header FontSize.
        /// </summary>
        public double HeaderFontSize { get; set; }

        /// <summary>
        /// Gets or sets Footer FontSize.
        /// </summary>
        public double FooterFontSize { get; set; }
    }
}
