﻿// <copyright file="TimeInfo.cs" company="GoalBook"> 
//    Copyright © 2009 Mark Brownsword. All rights reserved.
//    This source code and supporting files are licensed under The Code Project  
//    Open License (CPOL) as detailed at http://www.codeproject.com/info/cpol10.aspx. 
// </copyright>
namespace GoalBook.Infrastructure
{
    #region Using Statements
    using System;
    #endregion

    /// <summary>
    /// Time Info. Represents time of day.
    /// </summary>
    [Serializable] public struct TimeInfo
    {
        #region Constants
        #endregion

        #region Instance and Shared Fields
        #endregion

        #region Constructors
        /// <summary>
        /// Initializes a new instance of the TimeInfo struct.
        /// </summary>
        /// <param name="hour">hour parameter</param>
        /// <param name="minute">minute parameter</param>
        /// <param name="format">format parameter</param>
        public TimeInfo(string hour, string minute, ClockFormat format)
            : this()
        {
            this.Hour = hour;
            this.Minute = minute;
            this.Format = format;
        }
        #endregion

        #region Enums
        /// <summary>
        /// Declaration for ClockFormat enumeration.
        /// </summary>
        public enum ClockFormat 
        { 
            /// <summary>
            /// ante meridiem "before mid day"
            /// </summary>
            am, 
            
            /// <summary>
            /// post meridiem "after mid day" 
            /// </summary>
            pm 
        }
        #endregion

        #region Delegates and Events
        #endregion

        #region Properties
        /// <summary>
        /// Gets or sets a value indicating hour.
        /// </summary>
        public string Hour { get; set; }

        /// <summary>
        /// Gets or sets a value indicating minute.
        /// </summary>
        public string Minute { get; set; }

        /// <summary>
        /// Gets or sets a value indicating the Format.
        /// </summary>
        public ClockFormat? Format { get; set; }
        #endregion

        #region Public and internal Methods
        /// <summary>
        /// TryParse input as TwelveHourTimeInfo.
        /// </summary>
        /// <param name="input">input parameter</param>
        /// <param name="result">result parameter</param>
        /// <returns>true when parse is successful</returns>
        public static bool TryParse(string input, out TimeInfo result)
        {
            result = new TimeInfo();
            string[] splitResult = input.Split(
                new string[] { ":", " " }, 
                StringSplitOptions.RemoveEmptyEntries);

            switch (splitResult.Length)
            {                
                case 3: // 12 Hour format e.g. 3:30 pm
                    result.Hour = splitResult[0];
                    result.Minute = splitResult[1];
                    result.Format = (ClockFormat)Enum.Parse(
                        typeof(ClockFormat), 
                        splitResult[2]);

                    return true;                
            }

            return false;          
        }
        #endregion

        #region Base Class Overrides
        /// <summary>
        /// Get a string representation of the time.
        /// </summary>
        /// <returns>string representation of the time</returns>
        public override string ToString()
        {
            string result = string.Format("{0}:{1} {2}", this.Hour, this.Minute, this.Format);
            return result.Trim().Length == 7 || result.Trim().Length == 8 ? result : string.Empty;
        }
        #endregion

        #region Private and Protected Methods
        #endregion

        #region Event Handlers
        #endregion
    }
}
