﻿// <copyright file="HyperlinkDialogView.xaml.cs" company="GoalBook"> 
//    Copyright © 2009 Mark Brownsword. All rights reserved.
//    This source code and supporting files are licensed under The Code Project  
//    Open License (CPOL) as detailed at http://www.codeproject.com/info/cpol10.aspx. 
// </copyright>
namespace GoalBook.Notes.Views
{
    #region Using Statements
    using System;
    using System.ComponentModel;
    using System.Windows;
    using System.Windows.Controls;
    using GoalBook.Infrastructure.Events;
    using GoalBook.Infrastructure.Interfaces;
    #endregion
    
    /// <summary>
    /// Interaction logic for HyperlinkDialogView.xaml
    /// </summary>
    public partial class HyperlinkDialogView : UserControl, IErrorInfoDialogView
    {        
        #region Constants and Enums
        #endregion

        #region Instance and Shared Fields
        /// <summary>
        /// Declaration for ErrorInfoChangedEventHandler.
        /// </summary>
        private ErrorInfoChangedEventHandler errorInfoChangedEventHandler; 
        #endregion

        #region Constructors
        /// <summary>
        /// Initializes a new instance of the HyperlinkDialogView class.
        /// </summary>
        public HyperlinkDialogView()
        {
            InitializeComponent();
        }
        #endregion

        #region Delegates and Events
        /// <summary>
        /// Event indicating that the ErrorInfo property has changed.
        /// </summary>
        public event ErrorInfoChangedEventHandler ErrorInfoChanged
        {
            add { this.errorInfoChangedEventHandler = (ErrorInfoChangedEventHandler)Delegate.Combine(this.errorInfoChangedEventHandler, value); }
            remove { this.errorInfoChangedEventHandler = (ErrorInfoChangedEventHandler)Delegate.Remove(this.errorInfoChangedEventHandler, value); }
        }
        #endregion

        #region Properties
        #endregion

        #region Public and internal Methods
        /// <summary>
        /// Raise ErrorInfo Changed.
        /// </summary>
        /// <param name="info">IDataErrorInfo parameter</param>
        public void RaiseErrorInfoChanged(IDataErrorInfo info)
        {
            this.OnErrorInfoChanged(new ErrorInfoChangedEventArgs(info));
        }
        #endregion

        #region Base Class Overrides
        #endregion

        #region Private and Protected Methods
        /// <summary>
        /// On ErrorInfo Changed.
        /// </summary>
        /// <param name="args">ErrorInfoChangedEventArgs parameter</param>
        private void OnErrorInfoChanged(ErrorInfoChangedEventArgs args)
        {
            if (this.errorInfoChangedEventHandler != null)
            {
                this.errorInfoChangedEventHandler(args);
            }
        }
        #endregion
        
        #region Event Handlers
        /// <summary>
        /// Handle UserControl_Loaded event.
        /// </summary>
        /// <param name="sender">sender parameter</param>
        /// <param name="e">RoutedEventArgs parameter</param>
        private void UserControl_Loaded(object sender, RoutedEventArgs e)
        {
            this.Focus();
            this.textBoxUri.Focus(); 
        }
        #endregion
    }
}
