﻿// <copyright file="HyperlinkDialogViewPresenter.cs" company="GoalBook"> 
//    Copyright © 2009 Mark Brownsword. All rights reserved.
//    This source code and supporting files are licensed under The Code Project  
//    Open License (CPOL) as detailed at http://www.codeproject.com/info/cpol10.aspx. 
// </copyright>
namespace GoalBook.Notes.Views
{
    #region Using Statements
    using System.ComponentModel;
    using GoalBook.Infrastructure;
    using GoalBook.Infrastructure.Converters;
    using GoalBook.Infrastructure.Interfaces;
    #endregion

    /// <summary>
    /// Notes Dialog View Presenter class.
    /// </summary>
    public class HyperlinkDialogViewPresenter
    {
        #region Constants and Enums        
        #endregion

        #region Instance and Shared Fields
        #endregion

        #region Constructors
        /// <summary>
        /// Initializes a new instance of the HyperlinkDialogViewPresenter class.
        /// </summary> 
        /// <param name="hyperlinkInfo">HyperlinkInfo parameter</param>
        public HyperlinkDialogViewPresenter(HyperlinkInfo hyperlinkInfo)
        {
            this.Model = new HyperlinkDialogViewPresentationModel();
            this.Model.HyperlinkInfo = hyperlinkInfo;
            this.Model.HyperlinkInfo.PropertyChanged += this.HyperlinkInfo_PropertyChanged;
        }        
        #endregion

        #region Delegates and Events
        #endregion

        #region Properties
        /// <summary>
        /// Gets Model.
        /// </summary>
        public HyperlinkDialogViewPresentationModel Model { get; private set; }

        /// <summary>
        /// Gets or sets View.
        /// </summary>
        public HyperlinkDialogView View { get; set; }
        #endregion

        #region Public and internal Methods
        /// <summary>
        /// Initialise the View.
        /// </summary>
        public void InitView()
        {
            this.View.DataContext = this.Model;            
        }
        #endregion

        #region Base Class Overrides
        #endregion

        #region Private and Protected Methods          
        #endregion

        #region Event Handlers
        /// <summary>
        /// Handle HyperlinkInfo_PropertyChanged event.
        /// </summary>
        /// <param name="sender">sender parameter</param>
        /// <param name="e">PropertyChangedEventArgs parameter</param>
        private void HyperlinkInfo_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            this.View.RaiseErrorInfoChanged(this.Model.HyperlinkInfo as IDataErrorInfo);
        }
        #endregion
    }
}
