﻿//===============================================================================
// Goal Book.
// Copyright © 2009 Mark Brownsword. 
//===============================================================================

#region Using Statements
using System;
using System.ComponentModel;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using GoalBook.Infrastructure;
using GoalBook.Infrastructure.Interfaces;
using GoalBook.Shell.Misc;
using GoalBook.Shell.Windows;
using GoalBook.Shell.Views;
#endregion

namespace GoalBook.Shell.Services
{
    public class DialogService : IDialogService
    {        
        private readonly IShellView _shellView;
        
        /// <summary>
        /// Constructor.
        /// </summary>        
        public DialogService(IShellView shellView)
        {            
            _shellView = shellView;            
        }

        #region IDialogService Members
        /// <summary>
        /// ShowMessageDialog.
        /// </summary>        
        public bool? ShowMessageDialog(DialogOptions options)
        {
            Mouse.SetCursor(Cursors.Wait);
            DialogPresenter dialogPresenter = new DialogPresenter();            
            return dialogPresenter.ShowView(_shellView as Window, new MessageView(options.DialogMessage), options);
        }

        /// <summary>
        /// ShowView.
        /// </summary>        
        public bool? ShowView(UserControl view, DialogOptions options)
        {
            Mouse.SetCursor(Cursors.Wait);
            DialogPresenter dialogPresenter = new DialogPresenter();   
            return dialogPresenter.ShowView(_shellView as Window, view, options);
        }

        /// <summary>
        /// ShowView.
        /// </summary>        
        public bool? ShowView(UserControl view, DialogOptions options, Func<bool> validationAction)
        {
            Mouse.SetCursor(Cursors.Wait);
            DialogPresenter dialogPresenter = new DialogPresenter();
            return dialogPresenter.ShowView(_shellView as Window, view, options, validationAction);
        }

        /// <summary>
        /// ShowView.
        /// </summary>        
        public bool? ShowView(IErrorInfoDialogView view, DialogOptions options, IDataErrorInfo errorInfo)
        {
            Mouse.SetCursor(Cursors.Wait);
            DialogPresenter dialogPresenter = new DialogPresenter();
            return dialogPresenter.ShowView(_shellView as Window, view, options, errorInfo);
        }

        /// <summary>
        /// ShowView.
        /// </summary>        
        public bool? ShowView(IErrorInfoDialogView view, DialogOptions options, Func<bool> validationAction, IDataErrorInfo errorInfo)
        {
            Mouse.SetCursor(Cursors.Wait);
            DialogPresenter dialogPresenter = new DialogPresenter();
            return dialogPresenter.ShowView(_shellView as Window, view, options, validationAction, errorInfo);
        }      
        #endregion
    }
}
